import os
from typing import cast

import pytest
import pytest_asyncio

from beni import bcrypto
from beni.bsql import BModel, Db, MysqlDb

dbInfo = 'vjMwuuVArXJpEjGH6UPzZHwxgzA8AMTWCeEMeA/VoIzNwWwbmcQ3bwyYbVuCYMr0pLEO7j1D7YbOrYFERIWXAwu0FUsosFEINZRPCos666Qo1eOuJ48c07oUxUOQIt9HxwHCXEcmj8HCWzkFG9xvH5y45luZejIPVm4uLIdvw2okVhYyKIXOzYcLyucfj/6lYw=='
db = cast(Db, MysqlDb(**bcrypto.decryptJson(dbInfo, os.environ['XX_BENI'])))


@pytest_asyncio.fixture(scope='session', autouse=True)  # type: ignore
async def sessionSqlite():
    await initDb()
    yield


class Student(BModel):
    id: int = 0
    name: str = ''
    age: int = 0
    enabled: bool = False


async def initDb():
    async with db.getCursor() as cursor:
        await cursor.execute(f'DROP TABLE IF EXISTS `{Student.tableName()}`')
        await cursor.execute(f'''
            CREATE TABLE `{Student.tableName()}`  (
                `id` int NOT NULL AUTO_INCREMENT,
                `name` varchar(255) NULL,
                `age` int NULL,
                `enabled` tinyint(1) NOT NULL DEFAULT 0,
                PRIMARY KEY (`id`)
            );
        ''')
        await cursor.saveList([
            Student(name='A', age=21, enabled=True),
            Student(name='B', age=22),
            Student(name='C', age=23),
            Student(name='D', age=24),
        ])


@pytest.mark.order()
@pytest.mark.asyncio(scope='session')
async def test_addOne():
    student = Student(name='M', age=20, enabled=True)
    async with db.getCursor() as cursor:
        await cursor.addOne(student)
    async with db.getCursor() as cursor:
        result = await cursor.getOne(Student, f'SELECT * FROM {Student.tableName()} WHERE `name` = %s', student.name)
        assert result
        student.id = result.id
        assert result == student


@pytest.mark.order()
@pytest.mark.asyncio(scope='session')
async def test_saveOne():
    studentName = 'B'
    async with db.getCursor() as cursor:
        student = await cursor.getOne(Student, f'SELECT * FROM {Student.tableName()} WHERE `name` = %s', studentName)
        assert student
        newAge = student.age + 1
        student.age = newAge
        await cursor.saveOne(student)
    async with db.getCursor() as cursor:
        result = await cursor.getOne(Student, f'SELECT * FROM {Student.tableName()} WHERE `name` = %s', studentName)
        assert result and result.age == newAge


@pytest.mark.order()
@pytest.mark.asyncio(scope='session')
async def test_saveList():
    async with db.getCursor() as cursor:
        studentList = await cursor.getList(Student, f'SELECT * FROM {Student.tableName()} WHERE `name` IN (%x)', 'C', 'D')
        for student in studentList:
            student.enabled = True
        await cursor.saveList(studentList)
    async with db.getCursor() as cursor:
        studentList = await cursor.getList(Student, f'SELECT * FROM {Student.tableName()} WHERE `name` IN (%x)', 'C', 'D')
        assert {x.name: x.age for x in studentList} == {'C': 23, 'D': 24}


@pytest.mark.order()
@pytest.mark.asyncio(scope='session')
async def test_getTupleOne():
    studentName = 'A'
    async with db.getCursor() as cursor:
        result = await cursor.getTupleOne(f'SELECT * FROM {Student.tableName()} WHERE `name` = %s', studentName)
        assert result and result == (1, studentName, 21, True)


@pytest.mark.order()
@pytest.mark.asyncio(scope='session')
async def test_getDictOne():
    studentName = 'A'
    async with db.getCursor() as cursor:
        result = await cursor.getDictOne(f'SELECT * FROM {Student.tableName()} WHERE `name` = %s', studentName)
        assert result and result == {'id': 1, 'name': studentName, 'age': 21, 'enabled': True}


@pytest.mark.order()
@pytest.mark.asyncio(scope='session')
async def test_getOne():
    studentName = 'A'
    async with db.getCursor() as cursor:
        result = await cursor.getOne(Student, f'SELECT * FROM {Student.tableName()} WHERE `name` = %s', studentName)
        assert result
        user = Student(name=studentName, age=21, enabled=True)
        user.id = result.id
        assert user == result


@pytest.mark.order()
@pytest.mark.asyncio(scope='session')
async def test_getTupleList():
    async with db.getCursor() as cursor:
        resultList = await cursor.getTupleList(f'SELECT * FROM {Student.tableName()} WHERE `name` IN (%x)', 'A', 'B', 'C', 'D')
        assert len(resultList) == 4 and sorted([x[1] for x in resultList]) == ['A', 'B', 'C', 'D']


@pytest.mark.order()
@pytest.mark.asyncio(scope='session')
async def test_getDictList():
    async with db.getCursor() as cursor:
        resultList = await cursor.getDictList(f'SELECT * FROM {Student.tableName()} WHERE `name` IN (%x)', 'A', 'B', 'C', 'D')
        assert len(resultList) == 4 and sorted([x['name'] for x in resultList]) == ['A', 'B', 'C', 'D']


@pytest.mark.order()
@pytest.mark.asyncio(scope='session')
async def test_getList():
    async with db.getCursor() as cursor:
        resultList = await cursor.getList(Student, f'SELECT * FROM {Student.tableName()} WHERE `name` IN (%x)', 'A', 'B', 'C', 'D')
        assert len(resultList) == 4 and sorted([x.name for x in resultList]) == ['A', 'B', 'C', 'D']


@pytest.mark.order()
@pytest.mark.asyncio(scope='session')
async def test_getValue():
    async with db.getCursor() as cursor:
        result = await cursor.getValue(int, f'SELECT `age` FROM {Student.tableName()} WHERE `name` = %s', 'A')
        assert result and result == 21


@pytest.mark.order()
@pytest.mark.asyncio(scope='session')
async def test_getValueList():
    async with db.getCursor() as cursor:
        result = await cursor.getValueList(int, f'SELECT `age` FROM {Student.tableName()} WHERE `name` IN ( %x )', 'C', 'D')
        assert result and result == [23, 24]
